
      MODULE global_variables

      use lda_data
      use lda_interfaces

      implicit none

      real(8),parameter :: Econv=1.d-7     !  convergence (RPA Energy)
      real(8),parameter :: Hconv=1.d-6     !  convergence (Hartree)

      integer,parameter :: NEX=50
      character(30)     :: RPA_DATA(NEX)
      character(30)     :: RPA_INFO(NEX)
      character(30)     :: RPA_EF_DATA

      real(8),allocatable :: rpa1(:,:,:)   ! RPA wave function1
      real(8),allocatable :: rpa2(:,:,:)   ! RPA wave function2
      real(8),allocatable :: Fxc(:)        ! XC Kernel

      integer :: IC, OC, MEX0, MEX
      real(8) :: max_etime                 ! max elapsed time

      END MODULE global_variables

!================================================================== Main

      PROGRAM RPA
      use global_variables
      implicit none

      character(20) :: file_IN, SYS_NAME
      real(8)       :: En, fn(3)         ! energy, strength
      integer       :: m,n,i,j
      character(4)  :: XC_in

! Parameters

      read(*,*) SYS_NAME  ! Name of the system
      read(*,*) file_IN   ! LDA data file
      read(*,*) XC_in     ! XC kernle ( 'PZ' or 'VWN' )

      read(*,*) MEX0,MEX  ! calc. from the MEX0-th state
                          ! to the MEX-th excited state.

      read(*,*) OC        ! OC=1: write rpa wave function.
                          ! OC=2: write iteration info.
                          ! OC=3: write info. and w.f.

c Read LDA data

      open(4,file=file_IN,form='unformatted',status='old')
         call IN_data
      close(4)

c Allocate

      allocate( rpa1(ML,MST,MEX), rpa2(ML,MST,MEX), Fxc(ML) )
      rpa1=0.d0 ; rpa2=0.d0 ; Fxc=0.d0

c Exchange-Correlation kernel

      call Exc_Cor_Kernel(XC_in,Fxc)

      write(*,*) "XC Kernel = ",XC_in

c RPA data file name

      m=len_trim(SYS_NAME)

      n=0                   !  character        ASCII code
      Naming : do i=48,57   ! (0,1,...,9) --> (48,49,...,57)
      do j=48,57            !
         n=n+1 ; if ( n>NEX ) exit Naming
         RPA_DATA(n)=SYS_NAME(1:m)//"-rpa"//achar(i)//achar(j)//".data"
         RPA_INFO(n)=SYS_NAME(1:m)//"-rpa"//achar(i)//achar(j)//".info"
      end do
      end do Naming

      RPA_EF_DATA=SYS_NAME(1:m)//"-rpa"//".EFdat"

c Read RPA wave funcitons ( 1st ~ (MEX0-1)-th states )

      do n=1,MEX0-1
         open(4,file=RPA_DATA(n),form='unformatted',status='old')
            read(4) rpa1(:,:,n),rpa2(:,:,n)
         close(4)
      end do

c Timer ON

      call watch

c--------------- RPA ---------------

      do n=MEX0,MEX

         call Solve_RPAeq(n,En,fn)

c output

         open(1,file=RPA_EF_DATA,position='append')
            write(1,'(1x,i4,f12.5,3f10.5)') n,En,fn(1),fn(2),fn(3)
         close(1)

         if ( OC==1 .or. OC==3 ) then
            open(1,file=RPA_DATA(n),form='unformatted')
               write(1) rpa1(:,:,n),rpa2(:,:,n)
            close(1)
         end if

      end do

      stop

      END PROGRAM RPA

!=======================================================================
!============================================================ RPA solver

      SUBROUTINE Solve_RPAeq(n,En,fn)
      use global_variables
      implicit none

      integer,intent(IN)  :: n           ! n-th excited state
      real(8),intent(OUT) :: En, fn(3)   ! energy, strength

      integer,parameter :: maxiter = 1000
      character(30) :: RPA_DATA_TMP
      integer :: p,iter,iter0
      real(8) :: xAx,xBx,pAp,pBp,xAp,xBp,gg,gg_old,alpha,beta,c1,c2,c3
      real(8) :: En_old,dE,res,norm1,norm2,t
      real(8),allocatable :: g1(:,:),g2(:,:),p1(:,:),p2(:,:)

      allocate( g1(ML,MST),g2(ML,MST),p1(ML,MST),p2(ML,MST) )

      write(*,*) repeat("-",74)
      write(*,'(1x,"n =",i3)') n
      write(*,'(1x,a4,a12,3a10,a14,a14)')
     & "iter","      En    ","    fx    ","    fy    ","    fz    "
     &,"      dE      ","     res      "

c----- Initial RPA wave function -----

      call init_wf( rpa1(:,:,n), rpa2(:,:,n) )
      call ortho( rpa1(:,:,n), rpa2(:,:,n) )

      norm1=0.d0 ; norm2=0.d0
      do p=1,MST
         norm1 = norm1 + sum(rpa1(:,p,n)**2)*H**3
         norm2 = norm2 + sum(rpa2(:,p,n)**2)*H**3
      end do
      if ( norm1 > norm2 ) then
         do p=1,MST
            rpa1(:,p,n)=rpa1(:,p,n)/sqrt(norm1-norm2)
            rpa2(:,p,n)=rpa2(:,p,n)/sqrt(norm1-norm2)
         end do
      else
         do p=1,MST
            g1(:,p)=rpa1(:,p,n)
            rpa1(:,p,n)=rpa2(:,p,n)/sqrt(norm2-norm1)
            rpa2(:,p,n)=g1(:,p)/sqrt(norm2-norm1)
         end do
      end if

      iter0=1 ; gg_old=1.d50 ; En_old=0.d0
      p1=0.d0 ; p2=0.d0

!--------------------------------------------------------- C-G Iteration

      Iteration : do iter=iter0,maxiter

         call op_mat( rpa1(:,:,n),rpa2(:,:,n), g1,g2 )

         xAx=0.d0 ; xBx=0.d0
         do p=1,MST
            xAx=xAx+sum(rpa1(:,p,n)*g1(:,p))+sum(rpa2(:,p,n)*g2(:,p))
            xBx=xBx+sum(rpa1(:,p,n)**2)-sum(rpa2(:,p,n)**2)
         end do
         xAx=xAx*H**3 ; xBx=xBx*H**3 ; En = xAx/xBx

         do p=1,MST
            g1(:,p) = 2*( g1(:,p) - En*rpa1(:,p,n) ) / xBx
            g2(:,p) = 2*( g2(:,p) + En*rpa2(:,p,n) ) / xBx
         end do

         call ortho(g1,g2)

         gg=0.d0
         do p=1,MST
            gg=gg+sum(g1(:,p)**2)+sum(g2(:,p)**2)
         end do
         gg=gg*H**3 ; beta=gg/gg_old ; gg_old=gg
         p1 = -g1 + beta*p1 ; p2 = -g2 + beta*p2

         call op_mat( p1,p2, g1,g2 )
         pAp=0.d0 ; pBp=0.d0 ; xAp=0.d0 ; xBp=0.d0
         do p=1,MST
            pAp=pAp+sum(p1(:,p)*g1(:,p))+sum(p2(:,p)*g2(:,p))
            pBp=pBp+sum(p1(:,p)**2)-sum(p2(:,p)**2)
            xAp=xAp+sum(rpa1(:,p,n)*g1(:,p))+sum(rpa2(:,p,n)*g2(:,p))
            xBp=xBp+sum(rpa1(:,p,n)*p1(:,p))-sum(rpa2(:,p,n)*p2(:,p))
         end do
         pAp=pAp*H**3 ; pBp=pBp*H**3 ; xAp=xAp*H**3 ; xBp=xBp*H**3
         c1=pAp*xBp-xAp*pBp ; c2=pAp*xBx-xAx*pBp ; c3=xAp*xBx-xAx*xBp
         if ( c2**2-4*c1*c3 < 0.d0 ) then
            write(*,*) "D<0 !" ; alpha=0.5d0
         else
            alpha = ( -c2 + sqrt(c2**2-4*c1*c3) )/(2*c1)
         end if

         do p=1,MST
            rpa1(:,p,n) = rpa1(:,p,n) + alpha*p1(:,p)
            rpa2(:,p,n) = rpa2(:,p,n) + alpha*p2(:,p)
         end do

         call ortho(rpa1(:,:,n),rpa2(:,:,n))

c Iteration info.

         xBx=0.d0
         do p=1,MST
            xBx=xBx+sum(rpa1(:,p,n)**2)-sum(rpa2(:,p,n)**2)
         end do
         xBx=xBx*H**3 ; norm1=sqrt(abs(xBx))

         norm2=0.d0
         do p=1,MST
            norm2=norm2+sum(p1(:,p)**2)+sum(p2(:,p)**2)
         end do
         norm2=norm2*alpha*H**3

         res = norm2/xBx

         call strength(Lx(:ML)*H,fn(1))
         call strength(Ly(:ML)*H,fn(2))
         call strength(Lz(:ML)*H,fn(3))

         dE=abs(En-En_old)/(abs(En+En_old)*0.5d0) ; En_old=En
         write(*,'(1x,i4,f12.5,3f10.5,e14.5,e14.5)')
     &        iter,En,fn(1),fn(2),fn(3),dE,res

         if ( OC==2 .or. OC==3 ) then
            open(1,file=RPA_INFO(n),position='append')
               write(1,'(1x,i4,f12.5,3f10.5,e14.5,e14.5)')
     &              iter,En,fn(1),fn(2),fn(3),dE,res
            close(1)
         end if

c check the convergence

         if ( dE < Econv ) exit Iteration

      end do Iteration

!----------------------------------------------------- C-G Iteration END

c Normalize

      if ( xBx < 0.d0 ) stop " xBx < 0  (norm(rpa1) < norm(rpa2)) !!"

      norm1=sqrt(xBx)

      do p=1,MST
         rpa1(:,p,n)=rpa1(:,p,n)/norm1
         rpa2(:,p,n)=rpa2(:,p,n)/norm1
      end do

c Strength

      norm1=1.d0

      call strength(Lx(:ML)*H,fn(1))
      call strength(Ly(:ML)*H,fn(2))
      call strength(Lz(:ML)*H,fn(3))

      write(*,'(/1x,4f15.7)') En,fn(1),fn(2),fn(3) 

c CPU time

      call watch(DISP='on')

      deallocate( g1,g2,p1,p2 )

      return

      CONTAINS

      subroutine strength(V,f_0i)
      implicit none
      real(8),intent(IN)  :: V(ML)
      real(8),intent(OUT) :: f_0i
      real(8) :: f1,f2
      f1=0.d0 ; f2=0.d0
      do p=1,MST
         f1 = f1 + sum( V*rpa1(:,p,n)*psi(:,p) )
         f2 = f2 + sum( V*rpa2(:,p,n)*psi(:,p) )
      end do
      f_0i = 2*( (f1+f2)*H**3/norm1 )**2 / H2M * En
      end subroutine strength

!-----------------------------------------------------------------------

      subroutine op_mat(v1,v2,Av1,Av2)
      real(8),intent(IN)  :: v1(ML,MST) ,v2(ML,MST)
      real(8),intent(OUT) :: Av1(ML,MST),Av2(ML,MST)
      real(8),allocatable :: dn(:),dVh(:)
      integer :: p
      allocate( dn(ML),dVh(ML) ) ; dn=0.d0 ; dVh=0.d0
      do p=1,MST
         dn = dn + occ(p)*psi(:,p)*( v1(:,p) + v2(:,p) )
      end do
      call Hartree(dn,dVh,Hconv)
      do p=1,MST
         call hpsi(v1(:,p),Av1(:,p))
         Av1(:,p)=Av1(:,p)-esp(p)*v1(:,p)+psi(:,p)*(dVh+Fxc*dn)
         call hpsi(v2(:,p),Av2(:,p))
         Av2(:,p)=Av2(:,p)-esp(p)*v2(:,p)+psi(:,p)*(dVh+Fxc*dn)
      end do
      deallocate( dn,dVh )
      return
      end subroutine op_mat

!-----------------------------------------------------------------------

      subroutine ortho(v1,v2)
      real(8),intent(INOUT) :: v1(ML,MST),v2(ML,MST) 
      real(8) :: ovrp1(MST),ovrp2(MST),ovrp(MEX)
      integer :: p,m
      do p=1,MST
         ovrp1=matmul(v1(:,p),psi)*H**3
         ovrp2=matmul(v2(:,p),psi)*H**3
         v1(:,p) = v1(:,p) - matmul( psi,ovrp1 )
         v2(:,p) = v2(:,p) - matmul( psi,ovrp2 )
      end do
      ovrp=0.d0
      do m=1,n-1
         do p=1,MST
            ovrp(m) = ovrp(m) + sum(rpa1(:,p,m)*v1(:,p))*H**3
     &                        - sum(rpa2(:,p,m)*v2(:,p))*H**3
         end do
      end do
      do m=1,n-1
         do p=1,MST
            v1(:,p) = v1(:,p) - rpa1(:,p,m)*ovrp(m)
            v2(:,p) = v2(:,p) - rpa2(:,p,m)*ovrp(m)
         end do
      end do
      return
      end subroutine ortho 

      END SUBROUTINE Solve_RPAeq
