
      MODULE global_variables

      use lda_data
      use lda_interfaces

      implicit none

      integer       :: Ntime
      real(8)       :: dt,F
      character(1)  :: dir
      integer       :: Nenergy
      real(8)       :: dE
      character(30) :: file_RT
      character(30) :: file_alpha

      END MODULE global_variables

!=======================================================================

      PROGRAM Real_Time_DFT
      use global_variables
      implicit none

      character(30) :: file_IN
      real(8),allocatable :: Dp(:)
      real(8),allocatable :: alpha_R(:),alpha_I(:)
      integer :: n
      real(8) :: S

! Parameters

!      Ntime      = 2000                 ! Total time step
!      dt         = 0.01d0               ! Time step
!      F          = 0.01d0               ! Field strength
!      dir        = 'z'                  ! direction of the field
!      Nenergy    = 2000                 ! Energy range
!      dE         = 0.01                 ! Energy resolution
!      file_IN    ='C2H4-R5H3.data'      ! LDA input file name
!      file_RT    ='C2H4-R5H3-RT.data'   ! Real-Time data
!      file_alpha ='C2H4-R5H3-ALP.data'  ! Polarizability data

      read(*,*) Ntime
      read(*,*) dt
      read(*,*) F
      read(*,*) dir
      read(*,*) Nenergy
      read(*,*) dE
      read(*,*) file_IN
      read(*,*) file_RT
      read(*,*) file_alpha

      write(*,*)
      write(*,*) "Total time step      =",Ntime
      write(*,*) "Time step            =",dt
      write(*,*) "Field strength       =",F
      write(*,*) "      direction      =  ",dir
      write(*,*) "Energy range         =",Nenergy
      write(*,*) "Energy resolution    =",dE

      allocate( Dp(0:Ntime), alpha_R(0:Nenergy), alpha_I(0:Nenergy) )

c Read LDA data

      open(4,file=file_IN,form='unformatted',status='old')
         call IN_data
      close(4)
 
!--------------- Dipole moment ---------------

      call Time_Evolution(Dp)

      open(1,file=file_RT)
         do n=0,Ntime
            write(1,*) n*dt, Dp(n)
         end do
      close(1)

!---------- Polarizability and Strength ----------

      call Fourier_Transform

      open(1,file=file_alpha)
         do n=0,Nenergy
            S=2*n*dE/(Pi*a_B*E2**2)*alpha_I(n)
            write(1,*) n*dE, alpha_R(n), S
         end do
      close(1)

      stop

      CONTAINS

!----------------------------------------------------- Fourier transform

      subroutine Fourier_Transform
      complex(8),parameter ::  zi=(0.d0,1.d0)
      complex(8) :: zalpha
      integer :: m,n
      real(8) :: t,hw,TT
      TT = dt*Ntime
      do m=0,Nenergy
         hw=m*dE ; zalpha=(0.d0,0.d0)
         do n=1,Ntime
            t=n*dt ; zalpha=zalpha+exp(zi*hw*t)*Dp(n)
     &                            *(1-3*(t/TT)**2+2*(t/TT)**3)
         end do
         zalpha=zalpha*E2/F*dt        ! Complex polarizability
         alpha_R(m)=real(zalpha)      ! Real part
         alpha_I(m)=imag(zalpha)      ! Imaginary part
      end do

      end subroutine Fourier_Transform

      END PROGRAM Real_Time_DFT

!=======================================================================

      SUBROUTINE Time_Evolution(Dp)
      use global_variables
      implicit none

      real(8),intent(OUT)  :: Dp(0:Ntime)
      integer,parameter    :: N_hamil=4      !( order of oprator exp. )
      complex(8),parameter :: zi=(0.d0,1.d0)
      complex(8) :: zc(N_hamil),zpsi(ML,MST),htpsi(ML),tpsi(ML)
      integer :: m,n,t,p
      real(8) :: Ds,Ne,R(ML),Exc,Etot

! External field

      select case(dir)
         case('x') ; R=Lx(:ML)
         case('y') ; R=Ly(:ML)
         case('z') ; R=Lz(:ML)
      end select

! Static dipole moment

      Ds=sum(R*rho)*H**4

      write(*,*) "Static dipole moment =", Ds
      write(*,*)

c Taylor expansion coefficient

      do n=1,N_hamil
         zc(n)=(-zi*dt)**n
         do m=1,n
            zc(n)=zc(n)/m
         end do
      end do

! Initial wave function

      do p=1,MST
         zpsi(:,p)=exp(zi*F*R*H)*psi(:,p)
      end do

      Dp(0)=0.d0

!-------------------------------------------------- Time evolution

      write(*,'(1x,a6,a15,a15,a18)') "  time","  Dipole moment"
     &                  ,"      electrons","      Total energy"
      write(*,*) repeat('-',55)

      TE : do t=1,Ntime

         do p=1,MST                     ! Time evoluiton operator
            tpsi=zpsi(:,p)
            do n=1,N_hamil
               call hpsi(tpsi,htpsi)
               zpsi(:,p)=zpsi(:,p)+zc(n)*htpsi
               tpsi=htpsi
            end do
         end do

         rho = matmul( abs(zpsi)**2, occ )    ! Density

         call Hartree(rho,Vh,Hconv=1.d-10)    ! Hartree potential
         call Exc_Cor(XC,Exc)                 ! Exchange-Correlation

! result

         Dp(t)=sum(R*rho)*H**4 - Ds     ! Dipole moment
         Ne=sum(rho)*H**3               ! Number of electrons

         call Total_Energy              ! Total energy

         write(*,'(1x,i6,f15.8,f15.8,f18.8)') t,Dp(t),Ne,Etot

      end do TE

      CONTAINS

!---------------------------------------------------------- Total Energy

      subroutine Total_Energy
      integer    :: p,ia,ib
      real(8)    :: rab
      complex(8) :: Htpsi(ML)
      do p=1,MST
         call hpsi(zpsi(:,p),Htpsi)
         esp(p)=sum( conjg(zpsi(:,p))*Htpsi )*H**3
      end do
      Etot = sum( occ*esp ) + sum( rho*(-0.5d0*Vh-Vxc) )*H**3+Exc
      do ia=1,MI
         do ib=1,ia-1
            rab=sqrt( sum((Rion(:,ia)-Rion(:,ib))**2) )
            Etot=Etot+Zps(Kion(ia))*Zps(Kion(ib))*E2/rab
         end do
      end do
      end subroutine Total_Energy

      END SUBROUTINE Time_Evolution
